/**
 * Copyright 2007 Jordi Hernndez Sells, Matt Ruby
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License at
 * 
 * 	http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */
package net.jawr.web.resource.bundle.factory.util;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;

import net.jawr.web.resource.bundle.generator.GeneratorRegistry;
import net.jawr.web.servlet.JawrRequestHandler;

/**
 * Utility class to work with relative paths. 
 * 
 * @author Jordi Hernndez Sells
 * @author Matt Ruby
 *
 */
public class PathNormalizer {
	private static final String SEPARATOR = "/";
	
	/**
	 * Normalizes a bundle path mapping. If it ends with a wildcard, the wildcard is removed. 
	 * @param pathMapping
	 * @return
	 */
	public static final String normalizePathMapping(String pathMapping) {
		pathMapping = normalizePath(pathMapping);
		if(pathMapping.endsWith("/**"))
			pathMapping = pathMapping.substring(0,pathMapping.length()-3);
		return pathMapping;
	}
	
	/**
	 * Normalizes a path and adds a separator at its start. 
	 * @param path
	 * @return
	 */
	public static final String asPath(String path) {
		return(SEPARATOR + normalizePath(path));
	}
	
	/**
	 * Normalizes two paths and joins them as a single path. 
	 * @param prefix
	 * @param path
	 * @return
	 */
	public static final String joinPaths(String prefix,String path) {
		prefix = PathNormalizer.normalizePath(prefix);
		path = PathNormalizer.normalizePath(path);
		StringBuffer sb = new StringBuffer(SEPARATOR);
		if(!"".equals(prefix))
			sb.append(prefix).append(SEPARATOR);
		sb.append(path);
		
		return sb.toString();
		
	}
	
	/**
	 * Normalizes a dmoain name and a path and joins them as a single url. 
	 * @param domainName
	 * @param path
	 * @return
	 */
	public static final String joinDomainToPath(String domainName,String path) {
		if(domainName.endsWith("/")) {
			domainName = domainName.substring(0,domainName.length()-1);
		}
		path = PathNormalizer.normalizePath(path);
		StringBuffer sb = new StringBuffer(domainName);
		sb.append(SEPARATOR).append(path);
		
		return sb.toString();
		
	}	
	
	/**
	 * Removes leading and trailing separators from a path, and removes 
	 * double separators (// is replaced by /). 
	 * @param path
	 * @return
	 */
	public static final String normalizePath(String path) {
		path = path.replaceAll("//", SEPARATOR);
		StringTokenizer tk = new StringTokenizer(path,SEPARATOR);
		StringBuffer sb = new StringBuffer();
		while(tk.hasMoreTokens()) {
			sb.append(tk.nextToken());
			if(tk.hasMoreTokens())
				sb.append(SEPARATOR);
		}
		return sb.toString();
		
	}
	
	/**
	 * Normalizes all the paths in a Set. 
	 * @param paths
	 * @return
	 */
	public static final Set normalizePaths(Set paths) {
		Set ret = new HashSet();
		for(Iterator it = paths.iterator();it.hasNext();) {
			String path = normalizePath((String)it.next());
			ret.add(path);
		}
		return ret;
	}
	
	/**
	 * converts a generation path (such as jar:/some/path/file) into 
	 * a request path that the request handler can understand and process.  
	 * @param path
	 * @return
	 */
	public static String createGenerationPath(String path, GeneratorRegistry registry){
		try {
			path = registry.getDebugModeGenerationPath(path) 
				+ "?" 
				+ JawrRequestHandler.GENERATION_PARAM 
				+ "=" 
				+ URLEncoder.encode(path, "UTF-8");
		} catch (UnsupportedEncodingException neverHappens) {
			/*URLEncoder:how not to use checked exceptions...*/
			throw new RuntimeException("Something went unexpectedly wrong while encoding a URL for a generator. ",
										neverHappens);
		}
		return path;
	}
	
	/**
	 * Adds a key and value to the request path
	 * & or ? will be used as needed
	 * 
	 * path + ? or & + parameterKey=parameter
	 * 
	 * @param path the url to add the parameterKey and parameter too
	 * @param parameterKey the key in the get request (parameterKey=parameter)
	 * @param parameter the parameter to add to the end of the url
	 * @return a String with the url parameter added: path + ? or & + parameterKey=parameter
	 */
	public static String addGetParameter(String path, String parameterKey, String parameter){
		StringBuffer sb = new StringBuffer(path); 
		if(path.indexOf("?") > 0) {
			sb.append("&");
		} else {
			sb.append("?");
		}
		sb.append(parameterKey + "="+ parameter);
		return sb.toString();
	}
}
